@@ -57,7 +57,7 @@ track <- function(expr, pos=1, envir=as.environment(pos), list=NULL, pattern=NUL
                 paste("'", list[i], "'", sep="", collapse=", "))
         list <- list[-i]
     }
-    all.objs <- .Internal(ls(envir, TRUE))
+    all.objs <- ls(envir=envir, all.names=TRUE)
     for (objName in list) {
         ## Doesn't matter if it already exists....
         ## if (exists(objName, trackingEnv))
@@ -80,25 +80,7 @@ track <- function(expr, pos=1, envir=as.environment(pos), list=NULL, pattern=NUL
         }
         ## robustness: what to do if the assign inside setTrackedVar fails?
         setTrackedVar(objName, objval, trackingEnv, opt)
-        f <- substitute(function(v) {
-            if (missing(v))
-                getTrackedVar(x, envir)
-            else
-                setTrackedVar(x, v, envir)
-        }, list(x=objName, envir=trackingEnv))
-        mode(f) <- "function"
-        ## Need to replace the environment of f, otherwise it is this
-        ## function, which can contain a copy of objval, which can
-        ## use up lots of memory!
-        ## Need to be careful with the choice of env to set here:
-        ##   * emptyenv() doesn't work because then the binding can't find
-        ##     any defns
-        ##   * baseenv() doesn't work because then the function in the
-        ##     binding can't find functions from track
-        ##   * globalenv() doesn't work because the function in the
-        ##     binding can't find non-exported functions from track
-        ##   * parent.env(environment(f)) works!
-        environment(f) <- parent.env(environment(f))
+        f <- createBindingClosure(objName, trackingEnv)
         makeActiveBinding(objName, env=envir, fun=f)
     }
     return(invisible(list))
